# parseminsys.rb  -- parse the ITS MINSYS file into ruby code that
#                    can be run to make a DUMPER tape.
#
# written 010803 by Mirian Crzig Lennox
# Copyright (c) 2001, released under the terms of the GNU Public License.

require 'getoptlong.rb'
require 'pdp10.rb'

def helpmsg
  $stderr.print <<EOF
Usage: #{$0} [options...] minsysfile output.rb
Options:
  -r DIR --itsroot DIR   Root of dir where ITS files are found
  -h --help              This message

  minsysfile should be the file from AI: KSHACK; MINSYS >
    if it is omitted or '-', then stdin is used.
  outfile.rb is where the generated Ruby code will be written
    if it is omitted or '-', then stdout is used.
EOF
  exit(0)
end

def usage msg="syntax error, use --help to get help"
  $stderr.print "#{$0}: #{msg}\n"
  exit(1)
end

$itsroot = nil
$infile = nil
$outfile = nil

parser = GetoptLong.new
parser.set_options(
	['--itsroot', '-r', GetoptLong::REQUIRED_ARGUMENT],
	['--help', '-h', GetoptLong::NO_ARGUMENT])

begin
  parser.each do |optname, optarg|
    case optname
    when '--itsroot'
      if ($itsroot)
	usage "multiple --itsroot options"
      end
      $itsroot = optarg
    when '--help' then helpmsg
    end
  end
rescue
  usage
end
$itsroot = "." unless $itsroot

if (ARGV[0] and (ARGV[0] != '-'))
  $infile = File.open ARGV[0]
else
  $infile = $stdin
end

if (ARGV[1] and (ARGV[1] != '-'))
  $outfile = File.open ARGV[1], "w"
else
  $outfile = $stdout
end

include PDP10

def stringify(*arr)
  (arr.collect {|s| "\""+s+"\"" }).join(",")
end

def defile(dir,fn1,fn2,foo=nil,sdir=nil,sfn1=nil,sfn2=nil,comment=nil)
  sdir = dir unless (sdir and sdir != "")
  sfn1 = fn1 unless (sfn1 and sfn1 != "")
  sfn2 = fn2 unless (sfn2 and sfn2 != "")
  unixfn,unixext = its2unix($itsroot,sdir,sfn1,sfn2)
  unless unixfn
    $outfile.print "#tf.minfile(", stringify(dir,fn1,fn2,sdir,sfn1,sfn2), ")\n"
    $stderr.print "couldn't find file for #{dir}; #{fn1} #{fn2}\n"
    return
  end
  fn2 = unixext if fn2 == ">"
  unless File.exists? unixfn
    $outfile.print "#"
    $stderr.print "couldn't find \"#{unixfn}\" for #{dir}; #{fn1} #{fn2}\n"
  end
  $outfile.print "tf.minfile(", stringify(dir,fn1,fn2,unixfn), ")"
  if comment
    $outfile.print "   # ", comment
  end
  $outfile.print "\n"
end

def delink(dir,fn1,fn2,foo,sdir,sfn1,sfn2)
  defile(dir,fn1,fn2,foo,sdir,sfn1,sfn2,"link")
end

# linking isn't working, so make links be dumped files for now
#def delink(dir,fn1,fn2,foo,sdir,sfn1,sfn2)
#  sdir = dir unless (sdir and sdir != "")
#  sfn1 = fn1 unless (sfn1 and sfn1 != "")
#  sfn2 = fn2 unless (sfn2 and sfn2 != "")
#  $outfile.print "tf.minlink(", stringify(dir,fn1,fn2,sdir,sfn1,sfn2),")\n"
#end

$outfile.print <<EOF
# generated by #{$0} on #{Time.new}

require "minsys.rb"
tfn = ARGV[0]
unless tfn
  $stderr.print "usage: ", $0, " <tape-filename>\\n"
  $stderr.print "  <tape-filename> is the file which will be created\\n"
  exit(1)
end
tf = Minsys.new(tfn)
EOF

while (line = $infile.gets)
  line.chomp!
  if (line =~ /^(de(file|link))\s+/)
    method($1).call(*($'.split(',')))
  end
end

$infile.close unless $infile == $stdin

$outfile.print <<EOF
tf.close
EOF

$outfile.close unless $outfile == $stdout

